/*
 * File:        shadow.fs
 * Project:     Trailer 2006
 * Author:      Johanna Schmidt & Andreas Monitzer
 * Description: A fragment shader used for rendering using the shadow map (plus everything else).
 *
 * (c)2006 Johanna Schmidt, Andreas Monitzer. All Rights Reserved.
 */

uniform sampler2D texUnit0; // diffuse texture
uniform sampler2D normalMap; // normal map
uniform sampler2DShadow shadowMap0; // shadow map
uniform mat4 shadowMapMatrix0;
uniform mat4 shadowViewProjectionMatrix0;
uniform vec3 lightPosition0;
uniform float haveNormalmap;

// these are passed down from vertex shader
varying vec4 vpos;
varying vec3 normal;
varying vec3 view;

varying vec4 fpos_texspace;
varying vec3 normal_objspace;
varying vec3 normal_viewspace;

varying float depth;

void main(void) {
	const float fwidth = 0.1;
	float shadow = 0.0;
	vec4 shadowMapPos0 = (mat4(0.5,0.0,0.0,0.0,
	                           0.0,0.5,0.0,0.0,
	                           0.0,0.0,0.5,0.0,
						       0.5,0.5,0.5,1.0) * shadowMapMatrix0) * vpos;
	
	shadowMapPos0.xyzw /= shadowMapPos0.w;
	
	vec4 uniformshadowpos = shadowViewProjectionMatrix0 * vpos;
	shadowMapPos0.z = (uniformshadowpos.z) / uniformshadowpos.w; // workaround for polygon offset problem by the paper's authors
	
	shadow = shadow2D(shadowMap0, shadowMapPos0.xyz);
	
	vec2 texcoord;
	// ray/plane intersection test
	
	vec3 pointOnPlane = vec3(1.0,0.0,0.0);
	vec3 Xn = vec3(-1.0,0.0,0.0);
	vec3 N = normalize(normal_objspace);

	float XnDotN = dot(Xn, N);
	float t;
	
	// test with plane in x
	if(abs(XnDotN) > 0.5) {
		t = abs(dot(Xn, pointOnPlane - fpos_texspace.yxz) / XnDotN);

		texcoord = (fpos_texspace.yxz + N * t).yz;
	} else {
		// test with plane in y
		pointOnPlane = vec3(0.0,-1.0,0.0);
		Xn = vec3(0.0,1.0,0.0);
		XnDotN = dot(Xn, N);
		if(abs(XnDotN) > 0.1) {
			t = abs(dot(Xn, pointOnPlane - fpos_texspace.yxz) / XnDotN);
			
			texcoord = (fpos_texspace.yxz + N * t).xz;
		} else {
			// test with plane in z
			pointOnPlane = vec3(0.0,0.0,-1.0);
			Xn = vec3(0.0,0.0,1.0);
			t = abs(dot(Xn, pointOnPlane - fpos_texspace.yxz) / dot(Xn, N));

			texcoord = (fpos_texspace.yxz + N * t).xy;
		}
	}
	
	// modulate lighting with the computed shadow value
	vec4 color = texture2D(texUnit0, texcoord);
	
	vec3 lightVec = lightPosition0 - vpos.xyz;
		N = normalize(normal);
	vec3 L = normalize(lightVec);
	vec3 V = normalize(view);
	
	vec3 R = reflect(V, N);
	
	// calculate diffuse dot product
	float NdotL = max(dot(N, L), 0.0);
	
	if(color.a < 0.001)
		discard;

	gl_FragColor.rgb = (color.rgb * NdotL /*+ pow(max(dot(R, L), 0.0), 64.0)*/) * shadow + color.rgb * 0.1; //+ color.rgb * 0.4 * (1.0 - shadow);
	gl_FragColor.a = color.a;
}
