/*
 * File:        gameboard.vs
 * Project:     Trailer 2006
 * Author:      Johanna Schmidt & Andreas Monitzer
 * Description: A vertex shader used for rendering using the shadow map and parallax mapping.
 *
 * (c)2006 Johanna Schmidt, Andreas Monitzer. All Rights Reserved.
 */


uniform vec3 tangent;

varying vec4 vpos;
varying vec3 normal;
varying vec3 view;
varying vec3 normal_viewspace;
varying vec3 tview;
varying vec3 texCoord;


void main(void) {

	gl_TexCoord[0] = gl_TextureMatrix[0] * gl_MultiTexCoord0;
	gl_TexCoord[1] = gl_TextureMatrix[1] * gl_MultiTexCoord1;
	gl_TexCoord[2] = gl_TextureMatrix[2] * gl_MultiTexCoord2;
	gl_TexCoord[3] = gl_TextureMatrix[3] * gl_MultiTexCoord3;

	gl_Position = gl_ModelViewProjectionMatrix * gl_Vertex;
	
	vpos = gl_ModelViewMatrix * gl_Vertex;
	normal = gl_NormalMatrix * gl_Normal;
	view = (-vpos).xyz;

	normal_viewspace = normalize(gl_ModelViewProjectionMatrixInverseTranspose * vec4(gl_Normal,0.0));
	
	
	// for Parallax Mapping
	
	vec3 binormal = cross(tangent, gl_Normal); // Tangent Space
	vec3 eyeVector = normalize((gl_ModelViewMatrix * gl_Vertex).xyz);
	tview = normalize(vec3(dot(eyeVector, tangent), dot(eyeVector, binormal), dot(gl_Normal, -eyeVector)));
	
	texCoord = gl_MultiTexCoord0;
	
}
