/*
 * File:        relief.fs
 * Project:     Trailer 2006
 * Author:      Johanna Schmidt & Andreas Monitzer
 * Description: A fragment shader used for relief mapping.
 *
 * (c)2006 Johanna Schmidt, Andreas Monitzer. All Rights Reserved.
 */

uniform sampler2D texUnit0; // diffuse texture
uniform sampler2D normalMap; // normal map
uniform sampler2D heightMap; // height map
uniform vec3 lightPosition0;
uniform sampler2DShadow shadowMap0; // shadow map
uniform mat4 shadowMapMatrix0;
uniform mat4 shadowViewProjectionMatrix0;
// parameters for relief mapping:
uniform float scaleR;
uniform float biasR;

varying vec4 vpos;
varying vec3 view;
varying vec3 tview;
varying vec3 normal_viewspace;
varying vec3 texCoord;
varying vec3 normal;


void main(void) {
	
	vec3 viewV = normalize(tview);
	vec2 ds = viewV.xy / viewV.z;
	vec2 tempTC;
	int linear_steps = 10;
	float size = 1.0 / 10.0;
	float depth = 0.0;
	float best_depth = 1.0;
	float h = 0.0;	
	for (int i=0; i < linear_steps; i++) {
		depth = depth + size;
		if (best_depth > 0.996) {
			tempTC = texCoord.st + ( ((1.0 - depth) * scaleR + biasR) * viewV.xy);
			h = texture2D(heightMap, tempTC).x;
			if (depth >= h) {
				best_depth = depth;
			}
		}
	}
	depth = best_depth;

	for (int i=0; i < 10; i++) {
		size = size * 0.5;
		tempTC = texCoord.st + ( ((1.0 - depth) * scaleR + biasR) * viewV.xy);
		h = texture2D(heightMap, tempTC).x;
		if (depth >= h) {
			best_depth = depth;
			depth = depth - (2.0 * size);
		}
		depth = depth + size;
	}
	
	vec2 newTexCoord = texCoord.st + ( ((1.0 - best_depth) * scaleR + biasR) * viewV.xy);
	
	
	// Shadow Mapping

	const float fwidth = 0.1;
	float shadow = 0.0;
	vec4 shadowMapPos0 = (mat4(0.5,0.0,0.0,0.0,
	                           0.0,0.5,0.0,0.0,
	                           0.0,0.0,0.5,0.0,
						       0.5,0.5,0.5,1.0) * shadowMapMatrix0) * vpos;
	
	shadowMapPos0.xyzw /= shadowMapPos0.w;
	
	vec4 uniformshadowpos = shadowViewProjectionMatrix0 * vpos;
	shadowMapPos0.z = (uniformshadowpos.z) / uniformshadowpos.w; 
	// workaround for polygon offset problem by the paper's authors
	
	shadow = shadow2D(shadowMap0, shadowMapPos0.xyz);


	// Lighting using Normal Map
    
	vec3 N = normalize(gl_NormalMatrix * normal);
	vec3 V = normalize(view);
	vec3 R = reflect(V, N);

	vec3 lightVec = lightPosition0 - vpos.xyz;
	vec3 L = normalize(lightVec);
	
	vec3 eyeVec = vec3(0.0, 0.0, 1.0);
	vec3 halfVec = normalize(L + eyeVec);
	
	vec4 color = texture2D(texUnit0, newTexCoord.st);
	
	vec3 diffuse = vec3(max(.8*dot(N, L), .8)) * color.rgb;

	vec3 lobe = vec3(0.0,0.0,0.0);
	if(V.z > 0.0 && L.z > 0.0) {
		vec3 uv = V * L; // point-wise multiplication
		vec3 C;
		float n;
		
		// Lobe 1
		C = vec3(.86, .86, 0.77);
		n = 18.6;
		lobe += vec3(1.0,1.0,1.0) * 50.0 * pow(max(0.0,dot(C, uv)), n);

		// Lobe 2
		C = vec3(-0.41, -0.41, 0.018);
		n = 2.58;
		lobe += vec3(1.0,1.0,0.1) * 50.0 * pow(max(0.0,dot(C, uv)), n);

		// Lobe 3
		C = vec3(-1.03, -1.03, 0.7);
		n = 63.8;
		lobe += vec3(0.0,1.0,0.0) * 100.0 * pow(max(0.0,dot(C, uv)), n);
	}
	
	gl_FragColor.rgb = (diffuse + lobe) * shadow + vec3(.1);
	gl_FragColor.a = color.a;
	
}
