/*
 * File:        shadow.fs
 * Project:     Trailer 2006
 * Author:      Johanna Schmidt & Andreas Monitzer
 * Description: A fragment shader used for rendering using the shadow map (plus everything else).
 *
 * (c)2006 Johanna Schmidt, Andreas Monitzer. All Rights Reserved.
 */

uniform sampler2D texUnit0; // diffuse texture
uniform sampler2D normalMap; // normal map
uniform sampler2DShadow shadowMap0; // shadow map
uniform mat4 shadowMapMatrix0;
uniform mat4 shadowViewProjectionMatrix0;
uniform vec3 lightPosition0;
uniform float haveNormalmap;

// these are passed down from vertex shader
varying vec4 vpos;
varying vec3 normal;
varying vec3 view;

varying vec3 normal_viewspace;

varying float depth;

void main(void) {
	const float fwidth = 0.1;
	float shadow = 0.0;
	vec4 shadowMapPos0 = (mat4(0.5,0.0,0.0,0.0,
	                           0.0,0.5,0.0,0.0,
	                           0.0,0.0,0.5,0.0,
						       0.5,0.5,0.5,1.0) * shadowMapMatrix0) * vpos;
	
	shadowMapPos0.xyzw /= shadowMapPos0.w;
	
	vec4 uniformshadowpos = shadowViewProjectionMatrix0 * vpos;
	shadowMapPos0.z = (uniformshadowpos.z) / uniformshadowpos.w; // workaround for polygon offset problem by the paper's authors
	
	shadow = shadow2D(shadowMap0, shadowMapPos0.xyz);
	
	vec3 lightVec = lightPosition0 - vpos.xyz;
	vec3 N;
	if(haveNormalmap > 0.0) {
		N = texture2D(normalMap, gl_TexCoord[0].st).xzy;
		N.y = 1.0-N.y;
		N = normalize(gl_NormalMatrix * (N * 2.0 - 1.0));
	} else
		N = normalize(normal);
	vec3 L = normalize(lightVec);
	vec3 V = normalize(view);
	
	vec3 R = reflect(V, N);
	
	// calculate diffuse dot product
	float NdotL = max(dot(N, L), 0.0);
	
	// modulate lighting with the computed shadow value
	vec4 color = gl_Color;
	if(color.x > .999 && color.y > .999 && color.z > .999)
		color = texture2D(texUnit0, gl_TexCoord[0].st);
	
	if(color.a < 0.001)
		discard;
		
	gl_FragColor.rgb = (color.rgb * NdotL + pow(max(dot(R, L), 0.0), 64.0)) * shadow + color.rgb * 0.2; // + color.rgb * 0.4 * (1.0 - shadow);
	gl_FragColor.a = color.a;
}
